/* Sample TCP server */

#include <unistd.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define FILENAME_LEN 255


int main(int argc, char *argv[]) 
{ 
    int list_sock, accepted_sock; 
    int i,pid, n=0, clilen, verbose=1; 
    struct sockaddr_in srvaddr,cliaddr; 
    char file_to_serve[FILENAME_LEN]={0}; 
    time_t timeval; 
    int log=0, received, sent;

    /* check that enough commandline args have been passed */
    if(argc <1){
	printf("Poor man's FTP Server\nUsage: %s <log_file_name>\n",argv[0]);
	}

    /* open logfile*/
    log=open(argv[1],O_CREAT|O_RDWR,S_IRUSR|S_IWUSR);
    if(log <0){
	perror("cannot create logfile");
	exit(-1);
	}

    /* create socket */ 
    if ( (list_sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) { 
        perror("Socket creation error"); 
        exit(-1); 
    }

    /* initialize address */ 
    memset((void *)&srvaddr, 0, sizeof(srvaddr));     /*  clear server address structure*/ 
    srvaddr.sin_family = AF_INET;                     /*  address type is INET */ 
    srvaddr.sin_port = htons(2100);                   /*  we pick unprivileged port 2100 */ 
    srvaddr.sin_addr.s_addr = htonl(INADDR_ANY);      /*  Listen to connections from anywhere */ 

    /* bind to socket and start listen queue*/ 
    if (bind(list_sock, (struct sockaddr *)&srvaddr, sizeof(srvaddr)) < 0) { 
        perror("bind error"); 
        exit(-1); 
    } 
    listen(list_sock,10);
    
    printf("Bound to port 2100, entering daemon mode\n");
    daemon(1,0);
    dup2(log,STDOUT_FILENO);
    /* write daytime to client */ 
    while (1) {
        /* Accept a new client */
        accepted_sock = accept(list_sock, (struct sockaddr* ) &cliaddr, &clilen ); 
        if (verbose) {
            char text_ip_address[16]; 
            inet_ntop(AF_INET, &cliaddr.sin_addr, text_ip_address, sizeof(text_ip_address)); 
            printf("Host %s, port %d\n", text_ip_address, ntohs(cliaddr.sin_port)); 
	    fsync(log);
	} 
       
        pid = fork();       
        if (pid < 0) {
          perror("Issues in spawning listener child");
          return -1;
        }
 
        if(pid==0){
           close (list_sock);
           /* receive file description from client */
           received = 0;
           int last_recv = 0;
           /* receive name of file to be served */
           while (received < FILENAME_LEN){
              last_recv = recv(accepted_sock, file_to_serve+received, FILENAME_LEN-received-2,0);
              received = received + last_recv;
              if(file_to_serve[received-2] == '#' && file_to_serve[received-1] == '\n'){
                 file_to_serve[received-2] = '\0';
                 break;
              }
           }
          
 	   printf("Received request for file : %s \n",file_to_serve); 
           /* try to open file, and serve it */
           FILE* fp_to_serve;
           char  send_buffer[1024];
           int filled_buffer;
           fp_to_serve=fopen(file_to_serve,"r");
           if (fp_to_serve == NULL ){
             printf("could not open file %s",file_to_serve);
           } 
           while (!feof(fp_to_serve)){
             filled_buffer=fread(send_buffer,1024,1,fp_to_serve);
              printf(send_buffer);
              while(filled_buffer > 0){
              filled_buffer -=   send(accepted_sock,send_buffer,filled_buffer,0);
              } 
           }
           close(accepted_sock);
           exit(0);
  
        } else {
          /* close shared socket to avoid cross-listening */
          close(accepted_sock);
          
          int retval=0;
          wait(&retval);
        }
    } 
    /* should never get here during regular operation */ 
    exit(-1); 
} 
