/* Sample Daytime UDP server */

#include <unistd.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#define INPUT_BUFFER_LENGTH 80

int main(int argc, char *argv[]) 
{ 
    int sock; 
    int i, n=0, clilen, verbose=1; 
    struct sockaddr_in srvaddr,cliaddr; 
    char buffer[INPUT_BUFFER_LENGTH]={0}; 
    time_t timeval; 
    FILE* log;

    /* check that enough commandline args have been passed */
    if(argc <1){
	printf("Daytime Server\nUsage: %s <log_file_name>\n",argv[0]);
	}

    /* open logfile*/
    log=fopen(argv[1],"w+");
    if(log == NULL){
	perror("cannot create logfile");
	exit(-1);
	}

    /* create socket */ 
    if ( (sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) { 
        perror("Socket creation error"); 
        exit(-1); 
    }
    /* initialize address */ 
    memset((void *)&srvaddr, 0, sizeof(srvaddr));     /* clear server address structure*/ 
    srvaddr.sin_family = AF_INET;                  /* address type is INET */ 
    srvaddr.sin_port = htons(13);                  /* daytime port is 13 */ 
    srvaddr.sin_addr.s_addr = htonl(INADDR_ANY);   /* connect from anywhere */ 
    /* bind socket */ 
    if (bind(sock, (struct sockaddr *)&srvaddr, sizeof(srvaddr)) < 0) { 
        perror("bind error"); 
        exit(-1); 
    } 
    printf("Bound to port 13, entering daemon mode\n");
    daemon(1,0);
    /* write daytime to client */ 
    while (1) {
        n = recvfrom(sock, buffer, INPUT_BUFFER_LENGTH, 0, (struct sockaddr *)&cliaddr, &clilen); 
        if (n < 0) { 
            perror("recvfrom error"); 
            exit(-1); 
        } 
        if (verbose) { 
            inet_ntop(AF_INET, &cliaddr.sin_addr, buffer, sizeof(buffer)); 
            fprintf(log,"Request from host %s, port %d\n", buffer, ntohs(cliaddr.sin_port)); 
 	    fprintf(log,"Received request buffer : %s \n",buffer); 
	    fflush(log);
	} 
        timeval = time(NULL); 
        snprintf(buffer, sizeof(buffer), "%.24s\r\n", ctime(&timeval)); 
        n = sendto(sock, buffer, strlen(buffer), 0, (struct sockaddr *)&cliaddr, sizeof(cliaddr)); 
        if (n < 0) { 
            perror("sendto error"); 
            exit(-1);
        } 
    } 
    /* should never get here during regular operation */ 
    exit(-1); 
} 
