#include <unistd.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>


int main(int argc, char *argv[]) 
{ 
    int sock; 
    int i, n=0, clilen, verbose=1; 
    struct sockaddr_in6 srvaddr,cliaddr; 
    char buffer[80]={0}; 
    time_t timeval; 
    FILE* log;

    /* check that enough commandline args have been passed */
    if(argc <1){
	printf("Daytime Server\nUsage: %s <log_file_name>\n",argv[0]);
	}

    /* open logfile*/
    log=fopen(argv[1],"w+");
    if(log == NULL){
	perror("cannot create logfile");
	exit(-1);
	}
    
    /* create socket */ 
    if ( (sock = socket(AF_INET6, SOCK_DGRAM, 0)) < 0) { 
        perror("Socket creation error"); 
        exit(-1); 
    }
    /* initialize address */ 
    memset((void *)&srvaddr, 0, sizeof(srvaddr));     /* clear server address  */ 
    srvaddr.sin6_family = AF_INET6;                   /* address type is INET6 */ 
    srvaddr.sin6_port = htons(13);                    /* daytime port is 13    */ 
    srvaddr.sin6_addr = in6addr_any;     /* connect from anywhere */
    srvaddr.sin6_scope_id = 1;
    /* bind socket */ 
    if (bind(sock, (struct sockaddr *)&srvaddr, sizeof(srvaddr)) < 0) { 
        perror("bind error"); 
        exit(-1); 
    } 
    printf("Bound to port 13, entering daemon mode\n");
    daemon(1,0);
    /* write daytime to client */ 
    while (1) {
        n = recvfrom(sock, buffer, 80, 0, (struct sockaddr *)&cliaddr, &clilen); 
        if (n < 0) { 
            perror("recvfrom error"); 
            exit(-1); 
        } 
        if (verbose) { 
            inet_ntop(AF_INET6, &cliaddr.sin6_addr, buffer, sizeof(buffer)); 
            fprintf(log,"Request from host %s, port %d\n", buffer, ntohs(cliaddr.sin6_port)); 
	    fprintf(log,"buffer : %s \n",buffer);
	    fflush(log);

        } 
        timeval = time(NULL); 
        snprintf(buffer, sizeof(buffer), "%.24s\r\n", ctime(&timeval)); 
        n = sendto(sock, buffer, strlen(buffer), 0, (struct sockaddr *)&cliaddr, sizeof(cliaddr)); 
        if (n < 0) { 
            perror("sendto error"); 
            exit(-1);
        } 
    } 
    /* normal exit */ 
    exit(0); 
} 
